@@ -1,3 +1,20 @@
+/*
+   Copyright (c) 2016, 2017 MariaDB
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; version 2 of the License.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+
+#include "mariadb.h"
 #include "sql_class.h"
 #include "sql_lex.h"
 #include "sql_cte.h"
@@ -112,8 +129,8 @@ bool With_clause::check_dependencies()
          elem != with_elem;
          elem= elem->next)
     {
-      if (my_strcasecmp(system_charset_info, with_elem->query_name->str,
-                        elem->query_name->str) == 0)
+      if (lex_string_cmp(system_charset_info, with_elem->query_name,
+                         elem->query_name) == 0)
       {
 	my_error(ER_DUP_QUERY_NAME, MYF(0), with_elem->query_name->str);
 	return true;
@@ -223,7 +240,7 @@ With_element *With_clause::find_table_def(TABLE_LIST *table,
        with_elem= with_elem->next)
   {
     if (my_strcasecmp(system_charset_info, with_elem->query_name->str,
-		      table->table_name) == 0 &&
+		      table->table_name.str) == 0 &&
         !table->is_fqtn)
     {
       table->set_derived();
@@ -743,8 +760,7 @@ bool With_clause::prepare_unreferenced_elements(THD *thd)
 bool With_element::set_unparsed_spec(THD *thd, char *spec_start, char *spec_end)
 {
   unparsed_spec.length= spec_end - spec_start;
-  unparsed_spec.str= (char*) thd->memdup(spec_start, unparsed_spec.length+1);
-  unparsed_spec.str[unparsed_spec.length]= '\0';
+  unparsed_spec.str= thd->strmake(spec_start, unparsed_spec.length);
 
   if (!unparsed_spec.str)
   {
@@ -814,7 +830,7 @@ st_select_lex_unit *With_element::clone_parsed_spec(THD *thd,
   TABLE_LIST *spec_tables_tail;
   st_select_lex *with_select;
 
-  if (parser_state.init(thd, unparsed_spec.str, unparsed_spec.length))
+  if (parser_state.init(thd, (char*) unparsed_spec.str, (unsigned int)unparsed_spec.length))
     goto err;
   lex_start(thd);
   lex->stmt_lex= old_lex;
@@ -906,9 +922,9 @@ With_element::rename_columns_of_derived_unit(THD *thd,
   if (column_list.elements)  //  The column list is optional
   {
     List_iterator_fast<Item> it(select->item_list);
-    List_iterator_fast<LEX_STRING> nm(column_list);
+    List_iterator_fast<LEX_CSTRING> nm(column_list);
     Item *item;
-    LEX_STRING *name;
+    LEX_CSTRING *name;
 
     if (column_list.elements != select->item_list.elements)
     {
@@ -967,7 +983,7 @@ bool With_element::prepare_unreferenced(THD *thd)
 
   thd->lex->context_analysis_only|= CONTEXT_ANALYSIS_ONLY_DERIVED;
   if (!spec->prepared &&
-      (spec->prepare(thd, 0, 0) ||
+      (spec->prepare(spec->derived, 0, 0) ||
        rename_columns_of_derived_unit(thd, spec) ||
        check_duplicate_names(thd, first_sl->item_list, 1)))
     rc= true;
@@ -1310,32 +1326,29 @@ bool With_element::check_unrestricted_recursive(st_select_lex *sel,
 
 bool st_select_lex::check_subqueries_with_recursive_references()
 {
-  st_select_lex_unit *sl_master= master_unit();
   List_iterator<TABLE_LIST> ti(leaf_tables);
   TABLE_LIST *tbl;
   while ((tbl= ti++))
   {
-    if (!(tbl->is_with_table_recursive_reference() && sl_master->item))
+    if (!(tbl->is_with_table_recursive_reference()))
       continue;
-    With_element *with_elem= tbl->with;
-    bool check_embedding_materialized_derived= true;
+    With_element *rec_elem= tbl->with;
+    st_select_lex_unit *sl_master;
     for (st_select_lex *sl= this; sl; sl= sl_master->outer_select())
-    { 
+    {
       sl_master= sl->master_unit();
-      if (with_elem->get_owner() == sl_master->with_clause)
-         check_embedding_materialized_derived= false;
-      if (check_embedding_materialized_derived && !sl_master->with_element && 
-          sl_master->derived && sl_master->derived->is_materialized_derived())
+      if (sl_master->with_element &&
+          sl_master->with_element->get_owner() == rec_elem->get_owner())
+        break;
+      sl->uncacheable|= UNCACHEABLE_DEPENDENT;
+      sl_master->uncacheable|= UNCACHEABLE_DEPENDENT;
+      if (sl_master->derived)
+        sl_master->derived->register_as_derived_with_rec_ref(rec_elem);
+      if (sl_master->item)
       {
-	my_error(ER_REF_TO_RECURSIVE_WITH_TABLE_IN_DERIVED,
-	  	     MYF(0), with_elem->query_name->str);
-	return true;
+        Item_subselect *subq= (Item_subselect *) (sl_master->item);
+        subq->register_as_with_rec_ref(rec_elem);
       }
-      if (!sl_master->item)
-	continue;
-      Item_subselect *subq= (Item_subselect *) sl_master->item;
-      subq->with_recursive_reference= true;
-      subq->register_as_with_rec_ref(tbl->with);
     }
   }
   return false;
