@@ -1,8 +1,6 @@
 #include <my_global.h>
 #include <string.h>
 #include <m_ctype.h>
-
-
 #include "json_lib.h"
 
 /*
@@ -187,7 +185,7 @@ static int read_array(json_engine_t *j)
 /*
   Character classes inside the JSON string constant.
   We mostly need this to parse escaping properly.
-  Escapings availabe in JSON are:
+  Escapings available in JSON are:
   \" - quotation mark
   \\ - backslash
   \b - backspace UNICODE 8
@@ -1388,7 +1386,7 @@ int json_find_paths_next(json_engine_t *je, json_find_paths_t *state)
           if (!json_key_matches(je, &key_name))
             continue;
         }
-        if (cur_step - state->paths[p_c].last_step == state->cur_depth)
+        if ((uint) (cur_step - state->paths[p_c].last_step) == state->cur_depth)
           path_found= TRUE;
         else
         {
@@ -1421,7 +1419,7 @@ int json_find_paths_next(json_engine_t *je, json_find_paths_t *state)
             cur_step->n_item == state->array_counters[state->cur_depth])
         {
           /* Array item matches. */
-          if (cur_step - state->paths[p_c].last_step == state->cur_depth)
+          if ((uint) (cur_step - state->paths[p_c].last_step) == state->cur_depth)
             path_found= TRUE;
           else
           {
